﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Devart.Data.MySql;
using Devart.Data.MySql.Entity;

namespace MySqlFullTextSearch {

  class Program {

    static void Main(string[] args) {

      //using (var ctx = new MyContext()) {
      //  ctx.Database.ExecuteSqlCommand("DELETE FROM books");
      //}

      var monitor = new MySqlMonitor() { IsActive = true };

      UseSingleColumnIndex();
      UseMultipleColumnIndex();

      Console.ReadKey();
    }

    private static void UseSingleColumnIndex() {

      using (var ctx = new MyContext()) {

        // Load information into table Books
        ctx.Books.Add(new Book() {
          Author = "William Shakespeare",
          Name = "The Tragedy of King Richard the Second",
          Content = @"ACT 1
SCENE I. London. A Room in the palace.
[Enter KING RICHARD, attended; JOHN OF GAUNT, with other NOBLES.]
KING RICHARD.
  Old John of Gaunt, time-honoured Lancaster,
  Hast thou, according to thy oath and band,
  Brought hither Henry Hereford thy bold son,
  Here to make good the boisterous late appeal,
  Which then our leisure would not let us hear,
  Against the Duke of Norfolk, Thomas Mowbray?
..."
        });
        ctx.Books.Add(new Book() {
          Author = "William Shakespeare",
          Name = "The Life and Death of King John",
          Content = @"ACT 1.
SCENE 1. Northampton. A Room of State in the Palace.
[Enter KING JOHN, QUEEN ELINOR, PEMBROKE, ESSEX, SALISBURY, and others, with CHATILLON.]
KING JOHN.
  Now, say, Chatillon, what would France with us?
CHATILLON.
  Thus, after greeting, speaks the King of France,
  In my behaviour, to the majesty,
  The borrow'd majesty of England here.
..."
        });
        ctx.Books.Add(new Book() {
          Author = "William Shakespeare",
          Name = "The Life of Henry the Fifth",
          Content = @"ACT 1.
SCENE I. London. An ante-chamber in the King's palace.
[Enter the Archbishop of Canterbury and the Bishop of Ely.]
CANTERBURY.
  My lord, I'll tell you: that self bill is urg'd,
  Which in the eleventh year of the last king's reign
  Was like, and had indeed against us pass'd,
  But that the scambling and unquiet time
  Did push it out of farther question.
..."
        });
        ctx.SaveChanges();

        // Напишем аналог SQL запроса:
        //  SELECT id, name FROM books
        //  WHERE MATCH (content) AGAINST ('king palace Norfolk');
        var firstQuery = ctx.Books
          .Where(b => MySqlTextFunctions.MatchAgainstAsBool(b.Content, "king palace Norfolk"))
          .Select(b => new { b.Id, b.Name } );
        var firstQueryResults = firstQuery.ToList();

        // Output:
        Console.WriteLine();
        Console.WriteLine("== Querying your single column full-text index ==");
        Console.WriteLine();
        Console.WriteLine("1) Natural language");
        Console.WriteLine();
        Console.WriteLine("SQL:");
        Console.WriteLine(firstQuery.ToString());
        Console.WriteLine();
        Console.WriteLine("Data:");
        Console.WriteLine("{0,-5} | {1}", "Id", "Name");
        Console.WriteLine(new string('-', 50));
        foreach (var item in firstQueryResults)
          Console.WriteLine("{0,-5} | {1} ", item.Id, item.Name);        

        // Напишем аналог SQL запроса:
        //   SELECT id, name FROM books
        //   WHERE MATCH (content) AGAINST ('+king +palace -Norfolk' IN BOOLEAN MODE)
        var secondQuery = ctx.Books
                  .Where(b => MySqlTextFunctions.MatchAgainstInBooleanModeAsBool(b.Content, "+king +palace -Norfolk"))
                  .Select(b => new { b.Id, b.Name } );
        var secondQueryResults = secondQuery.ToList();

        // Output:
        Console.WriteLine();
        Console.WriteLine("2) Boolean mode");
        Console.WriteLine();
        Console.WriteLine("SQL:");
        Console.WriteLine(secondQuery.ToString());
        Console.WriteLine();
        Console.WriteLine("Data:");
        Console.WriteLine("{0,-5} | {1}", "Id", "Name");
        Console.WriteLine(new string('-', 50));
        foreach (var item in secondQueryResults)
          Console.WriteLine("{0,-5} | {1} ", item.Id, item.Name);
      }
    }

    private static void UseMultipleColumnIndex() {

      using (var ctx = new MyContext()) {

        // Напишем аналог SQL запроса:
        //   SELECT id, name FROM books
        //   where MATCH (Author, Name) AGAINST ('william king life death');
        var firstQuery = ctx.Books
          .Where(b => MySqlTextFunctions.MatchAgainstAsBool(b.Author, b.Name, "william king life death"))
          .Select(b => new { b.Id, b.Name });
        var firstQueryResults = firstQuery.ToList();

        // Output
        Console.WriteLine();
        Console.WriteLine("== Querying your multiple column full-text index ==");
        Console.WriteLine();
        Console.WriteLine("3) Natural language");
        Console.WriteLine();
        Console.WriteLine("SQL:");
        Console.WriteLine(firstQuery.ToString());
        Console.WriteLine();
        Console.WriteLine("Data:");
        Console.WriteLine("{0,-5} | {1}", "Id", "Name");
        Console.WriteLine(new string('-', 50));
        foreach (var item in firstQueryResults)
          Console.WriteLine("{0,-5} | {1} ", item.Id, item.Name);

        // Напишем аналог SQL запроса:
        //  SELECT b.*, MATCH (Author, Name) AGAINST ('william king life death' WITH QUERY EXPANSION)
        //  FROM books b WHERE MATCH (Author, Name) AGAINST ('william king life death' WITH QUERY EXPANSION) > 0.8
        var secondQuery = ctx.Books
                  .Where(b => MySqlTextFunctions.MatchAgainstWithQueryExpansion(b.Author, b.Name, "william king life death") > 0.8)
                  .Select(b => new {
                    Book = b,
                    Score = MySqlTextFunctions.MatchAgainstWithQueryExpansion(b.Author, b.Name, "william king life death")
                  });
        var secondQueryResults = secondQuery.ToList();

        // Output
        Console.WriteLine();
        Console.WriteLine("4) Natural language with query expansion");
        Console.WriteLine();
        Console.WriteLine("SQL:");
        Console.WriteLine(secondQuery.ToString());
        Console.WriteLine();
        Console.WriteLine("Data:");
        Console.WriteLine("{0,-5} | {1,-40} | {2}", "Id", "Name", "Score");
        Console.WriteLine(new string('-', 70));
        foreach (var item in secondQueryResults)
          Console.WriteLine("{0,-5} | {1,-40} | {2}", item.Book.Id, item.Book.Name, item.Score);

      }
    }

  }

}
